# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import (
    user_list_params,
    user_create_params,
    user_delete_params,
    user_update_params,
    user_retrieve_info_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    strip_not_given,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.user_create_response import UserCreateResponse

__all__ = ["UserResource", "AsyncUserResource"]


class UserResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return UserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return UserResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aliases: Optional[object] | NotGiven = NOT_GIVEN,
        allowed_cache_controls: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        auto_create_key: bool | NotGiven = NOT_GIVEN,
        blocked: Optional[bool] | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        config: Optional[object] | NotGiven = NOT_GIVEN,
        duration: Optional[str] | NotGiven = NOT_GIVEN,
        guardrails: Optional[List[str]] | NotGiven = NOT_GIVEN,
        key_alias: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        model_rpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        model_tpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        permissions: Optional[object] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        send_invite_email: Optional[bool] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        teams: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        user_alias: Optional[str] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        user_role: Optional[Literal["proxy_admin", "proxy_admin_viewer", "internal_user", "internal_user_viewer"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserCreateResponse:
        """Use this to create a new INTERNAL user with a budget.

        Internal Users can access
        LiteLLM Admin UI to make keys, request access to models. This creates a new user
        and generates a new api key for the new user. The new api key is returned.

        Returns user id, budget + new key.

        Parameters:

        - user_id: Optional[str] - Specify a user id. If not set, a unique id will be
          generated.
        - user_alias: Optional[str] - A descriptive name for you to know who this user
          id refers to.
        - teams: Optional[list] - specify a list of team id's a user belongs to.
        - user_email: Optional[str] - Specify a user email.
        - send_invite_email: Optional[bool] - Specify if an invite email should be sent.
        - user_role: Optional[str] - Specify a user role - "proxy_admin",
          "proxy_admin_viewer", "internal_user", "internal_user_viewer", "team",
          "customer". Info about each role here:
          `https://github.com/BerriAI/litellm/litellm/proxy/_types.py#L20`
        - max_budget: Optional[float] - Specify max budget for a given user.
        - budget_duration: Optional[str] - Budget is reset at the end of specified
          duration. If not set, budget is never reset. You can set duration as seconds
          ("30s"), minutes ("30m"), hours ("30h"), days ("30d"), months ("1mo").
        - models: Optional[list] - Model_name's a user is allowed to call. (if empty,
          key is allowed to call all models). Set to ['no-default-models'] to block all
          model access. Restricting user to only team-based model access.
        - tpm_limit: Optional[int] - Specify tpm limit for a given user (Tokens per
          minute)
        - rpm_limit: Optional[int] - Specify rpm limit for a given user (Requests per
          minute)
        - auto_create_key: bool - Default=True. Flag used for returning a key as part of
          the /user/new response
        - aliases: Optional[dict] - Model aliases for the user -
          [Docs](https://litellm.vercel.app/docs/proxy/virtual_keys#model-aliases)
        - config: Optional[dict] - [DEPRECATED PARAM] User-specific config.
        - allowed_cache_controls: Optional[list] - List of allowed cache control values.
          Example - ["no-cache", "no-store"]. See all values -
          https://docs.litellm.ai/docs/proxy/caching#turn-on--off-caching-per-request-
        - blocked: Optional[bool] - [Not Implemented Yet] Whether the user is blocked.
        - guardrails: Optional[List[str]] - [Not Implemented Yet] List of active
          guardrails for the user
        - permissions: Optional[dict] - [Not Implemented Yet] User-specific permissions,
          eg. turning off pii masking.
        - metadata: Optional[dict] - Metadata for user, store information for user.
          Example metadata = {"team": "core-infra", "app": "app2", "email":
          "ishaan@berri.ai" }
        - max_parallel_requests: Optional[int] - Rate limit a user based on the number
          of parallel requests. Raises 429 error, if user's parallel requests > x.
        - soft_budget: Optional[float] - Get alerts when user crosses given budget,
          doesn't block requests.
        - model_max_budget: Optional[dict] - Model-specific max budget for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-budgets-to-keys)
        - model_rpm_limit: Optional[float] - Model-specific rpm limit for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys)
        - model_tpm_limit: Optional[float] - Model-specific tpm limit for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys)
        - spend: Optional[float] - Amount spent by user. Default is 0. Will be updated
          by proxy whenever user is used. You can set duration as seconds ("30s"),
          minutes ("30m"), hours ("30h"), days ("30d"), months ("1mo").
        - team_id: Optional[str] - [DEPRECATED PARAM] The team id of the user. Default
          is None.
        - duration: Optional[str] - Duration for the key auto-created on `/user/new`.
          Default is None.
        - key_alias: Optional[str] - Alias for the key auto-created on `/user/new`.
          Default is None.

        Returns:

        - key: (str) The generated api key for the user
        - expires: (datetime) Datetime object for when key expires.
        - user_id: (str) Unique user id - used for tracking spend across multiple keys
          for same user id.
        - max_budget: (float|None) Max budget for given user.

        Usage Example

        ```shell
         curl -X POST "http://localhost:4000/user/new"      -H "Content-Type: application/json"      -H "Authorization: Bearer sk-1234"      -d '{
             "username": "new_user",
             "email": "new_user@example.com"
         }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/user/new",
            body=maybe_transform(
                {
                    "aliases": aliases,
                    "allowed_cache_controls": allowed_cache_controls,
                    "auto_create_key": auto_create_key,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "config": config,
                    "duration": duration,
                    "guardrails": guardrails,
                    "key_alias": key_alias,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "model_rpm_limit": model_rpm_limit,
                    "model_tpm_limit": model_tpm_limit,
                    "models": models,
                    "permissions": permissions,
                    "rpm_limit": rpm_limit,
                    "send_invite_email": send_invite_email,
                    "spend": spend,
                    "team_id": team_id,
                    "teams": teams,
                    "tpm_limit": tpm_limit,
                    "user_alias": user_alias,
                    "user_email": user_email,
                    "user_id": user_id,
                    "user_role": user_role,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCreateResponse,
        )

    def update(
        self,
        *,
        aliases: Optional[object] | NotGiven = NOT_GIVEN,
        allowed_cache_controls: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        blocked: Optional[bool] | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        config: Optional[object] | NotGiven = NOT_GIVEN,
        duration: Optional[str] | NotGiven = NOT_GIVEN,
        guardrails: Optional[List[str]] | NotGiven = NOT_GIVEN,
        key_alias: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        model_rpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        model_tpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        password: Optional[str] | NotGiven = NOT_GIVEN,
        permissions: Optional[object] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        user_role: Optional[Literal["proxy_admin", "proxy_admin_viewer", "internal_user", "internal_user_viewer"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Example curl

        ```
        curl --location 'http://0.0.0.0:4000/user/update'     --header 'Authorization: Bearer sk-1234'     --header 'Content-Type: application/json'     --data '{
            "user_id": "test-litellm-user-4",
            "user_role": "proxy_admin_viewer"
        }'
        ```

        Parameters: - user_id: Optional[str] - Specify a user id. If not set, a unique
        id will be generated. - user_email: Optional[str] - Specify a user email. -
        password: Optional[str] - Specify a user password. - user_alias: Optional[str] -
        A descriptive name for you to know who this user id refers to. - teams:
        Optional[list] - specify a list of team id's a user belongs to. -
        send_invite_email: Optional[bool] - Specify if an invite email should be sent. -
        user_role: Optional[str] - Specify a user role - "proxy_admin",
        "proxy_admin_viewer", "internal_user", "internal_user_viewer", "team",
        "customer". Info about each role here:
        `https://github.com/BerriAI/litellm/litellm/proxy/_types.py#L20` - max_budget:
        Optional[float] - Specify max budget for a given user. - budget_duration:
        Optional[str] - Budget is reset at the end of specified duration. If not set,
        budget is never reset. You can set duration as seconds ("30s"), minutes ("30m"),
        hours ("30h"), days ("30d"), months ("1mo"). - models: Optional[list] -
        Model_name's a user is allowed to call. (if empty, key is allowed to call all
        models) - tpm_limit: Optional[int] - Specify tpm limit for a given user (Tokens
        per minute) - rpm_limit: Optional[int] - Specify rpm limit for a given user
        (Requests per minute) - auto_create_key: bool - Default=True. Flag used for
        returning a key as part of the /user/new response - aliases: Optional[dict] -
        Model aliases for the user -
        [Docs](https://litellm.vercel.app/docs/proxy/virtual_keys#model-aliases) -
        config: Optional[dict] - [DEPRECATED PARAM] User-specific config. -
        allowed_cache_controls: Optional[list] - List of allowed cache control values.
        Example - ["no-cache", "no-store"]. See all values -
        https://docs.litellm.ai/docs/proxy/caching#turn-on--off-caching-per-request- -
        blocked: Optional[bool] - [Not Implemented Yet] Whether the user is blocked. -
        guardrails: Optional[List[str]] - [Not Implemented Yet] List of active
        guardrails for the user - permissions: Optional[dict] - [Not Implemented Yet]
        User-specific permissions, eg. turning off pii masking. - metadata:
        Optional[dict] - Metadata for user, store information for user. Example metadata
        = {"team": "core-infra", "app": "app2", "email": "ishaan@berri.ai" } -
        max_parallel_requests: Optional[int] - Rate limit a user based on the number of
        parallel requests. Raises 429 error, if user's parallel requests > x. -
        soft_budget: Optional[float] - Get alerts when user crosses given budget,
        doesn't block requests. - model_max_budget: Optional[dict] - Model-specific max
        budget for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-budgets-to-keys) -
        model_rpm_limit: Optional[float] - Model-specific rpm limit for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys) -
        model_tpm_limit: Optional[float] - Model-specific tpm limit for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys) -
        spend: Optional[float] - Amount spent by user. Default is 0. Will be updated by
        proxy whenever user is used. You can set duration as seconds ("30s"), minutes
        ("30m"), hours ("30h"), days ("30d"), months ("1mo"). - team_id: Optional[str] -
        [DEPRECATED PARAM] The team id of the user. Default is None. - duration:
        Optional[str] - [NOT IMPLEMENTED]. - key_alias: Optional[str] - [NOT
        IMPLEMENTED].

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/user/update",
            body=maybe_transform(
                {
                    "aliases": aliases,
                    "allowed_cache_controls": allowed_cache_controls,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "config": config,
                    "duration": duration,
                    "guardrails": guardrails,
                    "key_alias": key_alias,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "model_rpm_limit": model_rpm_limit,
                    "model_tpm_limit": model_tpm_limit,
                    "models": models,
                    "password": password,
                    "permissions": permissions,
                    "rpm_limit": rpm_limit,
                    "spend": spend,
                    "team_id": team_id,
                    "tpm_limit": tpm_limit,
                    "user_email": user_email,
                    "user_id": user_id,
                    "user_role": user_role,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        page: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        role: Optional[str] | NotGiven = NOT_GIVEN,
        user_ids: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get a paginated list of users, optionally filtered by role.

        Used by the UI to populate the user lists.

        Parameters: role: Optional[str] Filter users by role. Can be one of: -
        proxy_admin - proxy_admin_viewer - internal_user - internal_user_viewer
        user_ids: Optional[str] Get list of users by user_ids. Comma separated list of
        user_ids. page: int The page number to return page_size: int The number of items
        per page

        Currently - admin-only endpoint.

        Example curl:

        ```
        http://0.0.0.0:4000/user/list?user_ids=default_user_id,693c1a4a-1cc0-4c7c-afe8-b5d2c8d52e17
        ```

        Args:
          page: Page number

          page_size: Number of items per page

          role: Filter users by role

          user_ids: Get list of users by user_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/user/get_users",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "role": role,
                        "user_ids": user_ids,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=object,
        )

    def delete(
        self,
        *,
        user_ids: List[str],
        litellm_changed_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        delete user and associated user keys

        ```
        curl --location 'http://0.0.0.0:4000/user/delete'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data-raw '{
            "user_ids": ["45e3e396-ee08-4a61-a88e-16b3ce7e0849"]
        }'
        ```

        Parameters:

        - user_ids: List[str] - The list of user id's to be deleted.

        Args:
          litellm_changed_by: The litellm-changed-by header enables tracking of actions performed by
              authorized users on behalf of other users, providing an audit trail for
              accountability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"litellm-changed-by": litellm_changed_by}), **(extra_headers or {})}
        return self._post(
            "/user/delete",
            body=maybe_transform({"user_ids": user_ids}, user_delete_params.UserDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve_info(
        self,
        *,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [10/07/2024] Note: To get all users (+pagination), use `/user/list` endpoint.

        Use this to get user information. (user row + all user key info)

        Example request

        ```
        curl -X GET 'http://localhost:4000/user/info?user_id=krrish7%40berri.ai'     --header 'Authorization: Bearer sk-1234'
        ```

        Args:
          user_id: User ID in the request parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/user/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"user_id": user_id}, user_retrieve_info_params.UserRetrieveInfoParams),
            ),
            cast_to=object,
        )


class AsyncUserResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncUserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncUserResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aliases: Optional[object] | NotGiven = NOT_GIVEN,
        allowed_cache_controls: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        auto_create_key: bool | NotGiven = NOT_GIVEN,
        blocked: Optional[bool] | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        config: Optional[object] | NotGiven = NOT_GIVEN,
        duration: Optional[str] | NotGiven = NOT_GIVEN,
        guardrails: Optional[List[str]] | NotGiven = NOT_GIVEN,
        key_alias: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        model_rpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        model_tpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        permissions: Optional[object] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        send_invite_email: Optional[bool] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        teams: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        user_alias: Optional[str] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        user_role: Optional[Literal["proxy_admin", "proxy_admin_viewer", "internal_user", "internal_user_viewer"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserCreateResponse:
        """Use this to create a new INTERNAL user with a budget.

        Internal Users can access
        LiteLLM Admin UI to make keys, request access to models. This creates a new user
        and generates a new api key for the new user. The new api key is returned.

        Returns user id, budget + new key.

        Parameters:

        - user_id: Optional[str] - Specify a user id. If not set, a unique id will be
          generated.
        - user_alias: Optional[str] - A descriptive name for you to know who this user
          id refers to.
        - teams: Optional[list] - specify a list of team id's a user belongs to.
        - user_email: Optional[str] - Specify a user email.
        - send_invite_email: Optional[bool] - Specify if an invite email should be sent.
        - user_role: Optional[str] - Specify a user role - "proxy_admin",
          "proxy_admin_viewer", "internal_user", "internal_user_viewer", "team",
          "customer". Info about each role here:
          `https://github.com/BerriAI/litellm/litellm/proxy/_types.py#L20`
        - max_budget: Optional[float] - Specify max budget for a given user.
        - budget_duration: Optional[str] - Budget is reset at the end of specified
          duration. If not set, budget is never reset. You can set duration as seconds
          ("30s"), minutes ("30m"), hours ("30h"), days ("30d"), months ("1mo").
        - models: Optional[list] - Model_name's a user is allowed to call. (if empty,
          key is allowed to call all models). Set to ['no-default-models'] to block all
          model access. Restricting user to only team-based model access.
        - tpm_limit: Optional[int] - Specify tpm limit for a given user (Tokens per
          minute)
        - rpm_limit: Optional[int] - Specify rpm limit for a given user (Requests per
          minute)
        - auto_create_key: bool - Default=True. Flag used for returning a key as part of
          the /user/new response
        - aliases: Optional[dict] - Model aliases for the user -
          [Docs](https://litellm.vercel.app/docs/proxy/virtual_keys#model-aliases)
        - config: Optional[dict] - [DEPRECATED PARAM] User-specific config.
        - allowed_cache_controls: Optional[list] - List of allowed cache control values.
          Example - ["no-cache", "no-store"]. See all values -
          https://docs.litellm.ai/docs/proxy/caching#turn-on--off-caching-per-request-
        - blocked: Optional[bool] - [Not Implemented Yet] Whether the user is blocked.
        - guardrails: Optional[List[str]] - [Not Implemented Yet] List of active
          guardrails for the user
        - permissions: Optional[dict] - [Not Implemented Yet] User-specific permissions,
          eg. turning off pii masking.
        - metadata: Optional[dict] - Metadata for user, store information for user.
          Example metadata = {"team": "core-infra", "app": "app2", "email":
          "ishaan@berri.ai" }
        - max_parallel_requests: Optional[int] - Rate limit a user based on the number
          of parallel requests. Raises 429 error, if user's parallel requests > x.
        - soft_budget: Optional[float] - Get alerts when user crosses given budget,
          doesn't block requests.
        - model_max_budget: Optional[dict] - Model-specific max budget for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-budgets-to-keys)
        - model_rpm_limit: Optional[float] - Model-specific rpm limit for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys)
        - model_tpm_limit: Optional[float] - Model-specific tpm limit for user.
          [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys)
        - spend: Optional[float] - Amount spent by user. Default is 0. Will be updated
          by proxy whenever user is used. You can set duration as seconds ("30s"),
          minutes ("30m"), hours ("30h"), days ("30d"), months ("1mo").
        - team_id: Optional[str] - [DEPRECATED PARAM] The team id of the user. Default
          is None.
        - duration: Optional[str] - Duration for the key auto-created on `/user/new`.
          Default is None.
        - key_alias: Optional[str] - Alias for the key auto-created on `/user/new`.
          Default is None.

        Returns:

        - key: (str) The generated api key for the user
        - expires: (datetime) Datetime object for when key expires.
        - user_id: (str) Unique user id - used for tracking spend across multiple keys
          for same user id.
        - max_budget: (float|None) Max budget for given user.

        Usage Example

        ```shell
         curl -X POST "http://localhost:4000/user/new"      -H "Content-Type: application/json"      -H "Authorization: Bearer sk-1234"      -d '{
             "username": "new_user",
             "email": "new_user@example.com"
         }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/user/new",
            body=await async_maybe_transform(
                {
                    "aliases": aliases,
                    "allowed_cache_controls": allowed_cache_controls,
                    "auto_create_key": auto_create_key,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "config": config,
                    "duration": duration,
                    "guardrails": guardrails,
                    "key_alias": key_alias,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "model_rpm_limit": model_rpm_limit,
                    "model_tpm_limit": model_tpm_limit,
                    "models": models,
                    "permissions": permissions,
                    "rpm_limit": rpm_limit,
                    "send_invite_email": send_invite_email,
                    "spend": spend,
                    "team_id": team_id,
                    "teams": teams,
                    "tpm_limit": tpm_limit,
                    "user_alias": user_alias,
                    "user_email": user_email,
                    "user_id": user_id,
                    "user_role": user_role,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCreateResponse,
        )

    async def update(
        self,
        *,
        aliases: Optional[object] | NotGiven = NOT_GIVEN,
        allowed_cache_controls: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        blocked: Optional[bool] | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        config: Optional[object] | NotGiven = NOT_GIVEN,
        duration: Optional[str] | NotGiven = NOT_GIVEN,
        guardrails: Optional[List[str]] | NotGiven = NOT_GIVEN,
        key_alias: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        model_rpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        model_tpm_limit: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        password: Optional[str] | NotGiven = NOT_GIVEN,
        permissions: Optional[object] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        user_role: Optional[Literal["proxy_admin", "proxy_admin_viewer", "internal_user", "internal_user_viewer"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Example curl

        ```
        curl --location 'http://0.0.0.0:4000/user/update'     --header 'Authorization: Bearer sk-1234'     --header 'Content-Type: application/json'     --data '{
            "user_id": "test-litellm-user-4",
            "user_role": "proxy_admin_viewer"
        }'
        ```

        Parameters: - user_id: Optional[str] - Specify a user id. If not set, a unique
        id will be generated. - user_email: Optional[str] - Specify a user email. -
        password: Optional[str] - Specify a user password. - user_alias: Optional[str] -
        A descriptive name for you to know who this user id refers to. - teams:
        Optional[list] - specify a list of team id's a user belongs to. -
        send_invite_email: Optional[bool] - Specify if an invite email should be sent. -
        user_role: Optional[str] - Specify a user role - "proxy_admin",
        "proxy_admin_viewer", "internal_user", "internal_user_viewer", "team",
        "customer". Info about each role here:
        `https://github.com/BerriAI/litellm/litellm/proxy/_types.py#L20` - max_budget:
        Optional[float] - Specify max budget for a given user. - budget_duration:
        Optional[str] - Budget is reset at the end of specified duration. If not set,
        budget is never reset. You can set duration as seconds ("30s"), minutes ("30m"),
        hours ("30h"), days ("30d"), months ("1mo"). - models: Optional[list] -
        Model_name's a user is allowed to call. (if empty, key is allowed to call all
        models) - tpm_limit: Optional[int] - Specify tpm limit for a given user (Tokens
        per minute) - rpm_limit: Optional[int] - Specify rpm limit for a given user
        (Requests per minute) - auto_create_key: bool - Default=True. Flag used for
        returning a key as part of the /user/new response - aliases: Optional[dict] -
        Model aliases for the user -
        [Docs](https://litellm.vercel.app/docs/proxy/virtual_keys#model-aliases) -
        config: Optional[dict] - [DEPRECATED PARAM] User-specific config. -
        allowed_cache_controls: Optional[list] - List of allowed cache control values.
        Example - ["no-cache", "no-store"]. See all values -
        https://docs.litellm.ai/docs/proxy/caching#turn-on--off-caching-per-request- -
        blocked: Optional[bool] - [Not Implemented Yet] Whether the user is blocked. -
        guardrails: Optional[List[str]] - [Not Implemented Yet] List of active
        guardrails for the user - permissions: Optional[dict] - [Not Implemented Yet]
        User-specific permissions, eg. turning off pii masking. - metadata:
        Optional[dict] - Metadata for user, store information for user. Example metadata
        = {"team": "core-infra", "app": "app2", "email": "ishaan@berri.ai" } -
        max_parallel_requests: Optional[int] - Rate limit a user based on the number of
        parallel requests. Raises 429 error, if user's parallel requests > x. -
        soft_budget: Optional[float] - Get alerts when user crosses given budget,
        doesn't block requests. - model_max_budget: Optional[dict] - Model-specific max
        budget for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-budgets-to-keys) -
        model_rpm_limit: Optional[float] - Model-specific rpm limit for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys) -
        model_tpm_limit: Optional[float] - Model-specific tpm limit for user.
        [Docs](https://docs.litellm.ai/docs/proxy/users#add-model-specific-limits-to-keys) -
        spend: Optional[float] - Amount spent by user. Default is 0. Will be updated by
        proxy whenever user is used. You can set duration as seconds ("30s"), minutes
        ("30m"), hours ("30h"), days ("30d"), months ("1mo"). - team_id: Optional[str] -
        [DEPRECATED PARAM] The team id of the user. Default is None. - duration:
        Optional[str] - [NOT IMPLEMENTED]. - key_alias: Optional[str] - [NOT
        IMPLEMENTED].

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/user/update",
            body=await async_maybe_transform(
                {
                    "aliases": aliases,
                    "allowed_cache_controls": allowed_cache_controls,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "config": config,
                    "duration": duration,
                    "guardrails": guardrails,
                    "key_alias": key_alias,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "model_rpm_limit": model_rpm_limit,
                    "model_tpm_limit": model_tpm_limit,
                    "models": models,
                    "password": password,
                    "permissions": permissions,
                    "rpm_limit": rpm_limit,
                    "spend": spend,
                    "team_id": team_id,
                    "tpm_limit": tpm_limit,
                    "user_email": user_email,
                    "user_id": user_id,
                    "user_role": user_role,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        page: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        role: Optional[str] | NotGiven = NOT_GIVEN,
        user_ids: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get a paginated list of users, optionally filtered by role.

        Used by the UI to populate the user lists.

        Parameters: role: Optional[str] Filter users by role. Can be one of: -
        proxy_admin - proxy_admin_viewer - internal_user - internal_user_viewer
        user_ids: Optional[str] Get list of users by user_ids. Comma separated list of
        user_ids. page: int The page number to return page_size: int The number of items
        per page

        Currently - admin-only endpoint.

        Example curl:

        ```
        http://0.0.0.0:4000/user/list?user_ids=default_user_id,693c1a4a-1cc0-4c7c-afe8-b5d2c8d52e17
        ```

        Args:
          page: Page number

          page_size: Number of items per page

          role: Filter users by role

          user_ids: Get list of users by user_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/user/get_users",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "role": role,
                        "user_ids": user_ids,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=object,
        )

    async def delete(
        self,
        *,
        user_ids: List[str],
        litellm_changed_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        delete user and associated user keys

        ```
        curl --location 'http://0.0.0.0:4000/user/delete'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data-raw '{
            "user_ids": ["45e3e396-ee08-4a61-a88e-16b3ce7e0849"]
        }'
        ```

        Parameters:

        - user_ids: List[str] - The list of user id's to be deleted.

        Args:
          litellm_changed_by: The litellm-changed-by header enables tracking of actions performed by
              authorized users on behalf of other users, providing an audit trail for
              accountability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"litellm-changed-by": litellm_changed_by}), **(extra_headers or {})}
        return await self._post(
            "/user/delete",
            body=await async_maybe_transform({"user_ids": user_ids}, user_delete_params.UserDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve_info(
        self,
        *,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [10/07/2024] Note: To get all users (+pagination), use `/user/list` endpoint.

        Use this to get user information. (user row + all user key info)

        Example request

        ```
        curl -X GET 'http://localhost:4000/user/info?user_id=krrish7%40berri.ai'     --header 'Authorization: Bearer sk-1234'
        ```

        Args:
          user_id: User ID in the request parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/user/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"user_id": user_id}, user_retrieve_info_params.UserRetrieveInfoParams
                ),
            ),
            cast_to=object,
        )


class UserResourceWithRawResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.create = to_raw_response_wrapper(
            user.create,
        )
        self.update = to_raw_response_wrapper(
            user.update,
        )
        self.list = to_raw_response_wrapper(
            user.list,
        )
        self.delete = to_raw_response_wrapper(
            user.delete,
        )
        self.retrieve_info = to_raw_response_wrapper(
            user.retrieve_info,
        )


class AsyncUserResourceWithRawResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.create = async_to_raw_response_wrapper(
            user.create,
        )
        self.update = async_to_raw_response_wrapper(
            user.update,
        )
        self.list = async_to_raw_response_wrapper(
            user.list,
        )
        self.delete = async_to_raw_response_wrapper(
            user.delete,
        )
        self.retrieve_info = async_to_raw_response_wrapper(
            user.retrieve_info,
        )


class UserResourceWithStreamingResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.create = to_streamed_response_wrapper(
            user.create,
        )
        self.update = to_streamed_response_wrapper(
            user.update,
        )
        self.list = to_streamed_response_wrapper(
            user.list,
        )
        self.delete = to_streamed_response_wrapper(
            user.delete,
        )
        self.retrieve_info = to_streamed_response_wrapper(
            user.retrieve_info,
        )


class AsyncUserResourceWithStreamingResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.create = async_to_streamed_response_wrapper(
            user.create,
        )
        self.update = async_to_streamed_response_wrapper(
            user.update,
        )
        self.list = async_to_streamed_response_wrapper(
            user.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            user.delete,
        )
        self.retrieve_info = async_to_streamed_response_wrapper(
            user.retrieve_info,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["KeyListParams"]


class KeyListParams(TypedDict, total=False):
    include_team_keys: bool
    """Include all keys for teams that user is an admin of."""

    key_alias: Optional[str]
    """Filter keys by key alias"""

    organization_id: Optional[str]
    """Filter keys by organization ID"""

    page: int
    """Page number"""

    return_full_object: bool
    """Return full key object"""

    size: int
    """Page size"""

    team_id: Optional[str]
    """Filter keys by team ID"""

    user_id: Optional[str]
    """Filter keys by user ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .user_roles import UserRoles

__all__ = ["OrganizationUpdateMemberParams"]


class OrganizationUpdateMemberParams(TypedDict, total=False):
    organization_id: Required[str]

    max_budget_in_organization: Optional[float]

    role: Optional[UserRoles]
    """
    Admin Roles: PROXY_ADMIN: admin over the platform PROXY_ADMIN_VIEW_ONLY: can
    login, view all own keys, view all spend ORG_ADMIN: admin over a specific
    organization, can create teams, users only within their organization

    Internal User Roles: INTERNAL_USER: can login, view/create/delete their own
    keys, view their spend INTERNAL_USER_VIEW_ONLY: can login, view their own keys,
    view their own spend

    Team Roles: TEAM: used for JWT auth

    Customer Roles: CUSTOMER: External users -> these are customers
    """

    user_email: Optional[str]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types.global_ import (
    SpendListTagsResponse,
    SpendRetrieveReportResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpend:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_tags(self, client: Hanzo) -> None:
        spend = client.global_.spend.list_tags()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_tags_with_all_params(self, client: Hanzo) -> None:
        spend = client.global_.spend.list_tags(
            end_date="end_date",
            start_date="start_date",
            tags="tags",
        )
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_tags(self, client: Hanzo) -> None:
        response = client.global_.spend.with_raw_response.list_tags()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_tags(self, client: Hanzo) -> None:
        with client.global_.spend.with_streaming_response.list_tags() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(SpendListTagsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_reset(self, client: Hanzo) -> None:
        spend = client.global_.spend.reset()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_reset(self, client: Hanzo) -> None:
        response = client.global_.spend.with_raw_response.reset()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_reset(self, client: Hanzo) -> None:
        with client.global_.spend.with_streaming_response.reset() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(object, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_report(self, client: Hanzo) -> None:
        spend = client.global_.spend.retrieve_report()
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_report_with_all_params(self, client: Hanzo) -> None:
        spend = client.global_.spend.retrieve_report(
            api_key="api_key",
            customer_id="customer_id",
            end_date="end_date",
            group_by="team",
            internal_user_id="internal_user_id",
            start_date="start_date",
            team_id="team_id",
        )
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_report(self, client: Hanzo) -> None:
        response = client.global_.spend.with_raw_response.retrieve_report()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_report(self, client: Hanzo) -> None:
        with client.global_.spend.with_streaming_response.retrieve_report() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSpend:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_tags(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.global_.spend.list_tags()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_tags_with_all_params(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.global_.spend.list_tags(
            end_date="end_date",
            start_date="start_date",
            tags="tags",
        )
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_tags(self, async_client: AsyncHanzo) -> None:
        response = await async_client.global_.spend.with_raw_response.list_tags()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_tags(self, async_client: AsyncHanzo) -> None:
        async with async_client.global_.spend.with_streaming_response.list_tags() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(SpendListTagsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_reset(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.global_.spend.reset()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_reset(self, async_client: AsyncHanzo) -> None:
        response = await async_client.global_.spend.with_raw_response.reset()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_reset(self, async_client: AsyncHanzo) -> None:
        async with async_client.global_.spend.with_streaming_response.reset() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(object, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_report(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.global_.spend.retrieve_report()
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_report_with_all_params(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.global_.spend.retrieve_report(
            api_key="api_key",
            customer_id="customer_id",
            end_date="end_date",
            group_by="team",
            internal_user_id="internal_user_id",
            start_date="start_date",
            team_id="team_id",
        )
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_report(self, async_client: AsyncHanzo) -> None:
        response = await async_client.global_.spend.with_raw_response.retrieve_report()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_report(self, async_client: AsyncHanzo) -> None:
        async with async_client.global_.spend.with_streaming_response.retrieve_report() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(SpendRetrieveReportResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cache import (
    CacheResource,
    AsyncCacheResource,
    CacheResourceWithRawResponse,
    AsyncCacheResourceWithRawResponse,
    CacheResourceWithStreamingResponse,
    AsyncCacheResourceWithStreamingResponse,
)
from .redis import (
    RedisResource,
    AsyncRedisResource,
    RedisResourceWithRawResponse,
    AsyncRedisResourceWithRawResponse,
    RedisResourceWithStreamingResponse,
    AsyncRedisResourceWithStreamingResponse,
)

__all__ = [
    "RedisResource",
    "AsyncRedisResource",
    "RedisResourceWithRawResponse",
    "AsyncRedisResourceWithRawResponse",
    "RedisResourceWithStreamingResponse",
    "AsyncRedisResourceWithStreamingResponse",
    "CacheResource",
    "AsyncCacheResource",
    "CacheResourceWithRawResponse",
    "AsyncCacheResourceWithRawResponse",
    "CacheResourceWithStreamingResponse",
    "AsyncCacheResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .member import Member
from .._models import BaseModel
from .lite_llm_model_table import LiteLlmModelTable

__all__ = ["LiteLlmTeamTable"]


class LiteLlmTeamTable(BaseModel):
    team_id: str

    admins: Optional[List[object]] = None

    blocked: Optional[bool] = None

    budget_duration: Optional[str] = None

    budget_reset_at: Optional[datetime] = None

    created_at: Optional[datetime] = None

    litellm_model_table: Optional[LiteLlmModelTable] = None

    max_budget: Optional[float] = None

    max_parallel_requests: Optional[int] = None

    members: Optional[List[object]] = None

    members_with_roles: Optional[List[Member]] = None

    metadata: Optional[object] = None

    api_model_id: Optional[int] = FieldInfo(alias="model_id", default=None)

    models: Optional[List[object]] = None

    organization_id: Optional[str] = None

    rpm_limit: Optional[int] = None

    spend: Optional[float] = None

    team_alias: Optional[str] = None

    tpm_limit: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .add import (
    AddResource,
    AsyncAddResource,
    AddResourceWithRawResponse,
    AsyncAddResourceWithRawResponse,
    AddResourceWithStreamingResponse,
    AsyncAddResourceWithStreamingResponse,
)
from .key import (
    KeyResource,
    AsyncKeyResource,
    KeyResourceWithRawResponse,
    AsyncKeyResourceWithRawResponse,
    KeyResourceWithStreamingResponse,
    AsyncKeyResourceWithStreamingResponse,
)
from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .team import (
    TeamResource,
    AsyncTeamResource,
    TeamResourceWithRawResponse,
    AsyncTeamResourceWithRawResponse,
    TeamResourceWithStreamingResponse,
    AsyncTeamResourceWithStreamingResponse,
)
from .test import (
    TestResource,
    AsyncTestResource,
    TestResourceWithRawResponse,
    AsyncTestResourceWithRawResponse,
    TestResourceWithStreamingResponse,
    AsyncTestResourceWithStreamingResponse,
)
from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from .audio import (
    AudioResource,
    AsyncAudioResource,
    AudioResourceWithRawResponse,
    AsyncAudioResourceWithRawResponse,
    AudioResourceWithStreamingResponse,
    AsyncAudioResourceWithStreamingResponse,
)
from .azure import (
    AzureResource,
    AsyncAzureResource,
    AzureResourceWithRawResponse,
    AsyncAzureResourceWithRawResponse,
    AzureResourceWithStreamingResponse,
    AsyncAzureResourceWithStreamingResponse,
)
from .cache import (
    CacheResource,
    AsyncCacheResource,
    CacheResourceWithRawResponse,
    AsyncCacheResourceWithRawResponse,
    CacheResourceWithStreamingResponse,
    AsyncCacheResourceWithStreamingResponse,
)
from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .model import (
    ModelResource,
    AsyncModelResource,
    ModelResourceWithRawResponse,
    AsyncModelResourceWithRawResponse,
    ModelResourceWithStreamingResponse,
    AsyncModelResourceWithStreamingResponse,
)
from .spend import (
    SpendResource,
    AsyncSpendResource,
    SpendResourceWithRawResponse,
    AsyncSpendResourceWithRawResponse,
    SpendResourceWithStreamingResponse,
    AsyncSpendResourceWithStreamingResponse,
)
from .utils import (
    UtilsResource,
    AsyncUtilsResource,
    UtilsResourceWithRawResponse,
    AsyncUtilsResourceWithRawResponse,
    UtilsResourceWithStreamingResponse,
    AsyncUtilsResourceWithStreamingResponse,
)
from .active import (
    ActiveResource,
    AsyncActiveResource,
    ActiveResourceWithRawResponse,
    AsyncActiveResourceWithRawResponse,
    ActiveResourceWithStreamingResponse,
    AsyncActiveResourceWithStreamingResponse,
)
from .budget import (
    BudgetResource,
    AsyncBudgetResource,
    BudgetResourceWithRawResponse,
    AsyncBudgetResourceWithRawResponse,
    BudgetResourceWithStreamingResponse,
    AsyncBudgetResourceWithStreamingResponse,
)
from .cohere import (
    CohereResource,
    AsyncCohereResource,
    CohereResourceWithRawResponse,
    AsyncCohereResourceWithRawResponse,
    CohereResourceWithStreamingResponse,
    AsyncCohereResourceWithStreamingResponse,
)
from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from .delete import (
    DeleteResource,
    AsyncDeleteResource,
    DeleteResourceWithRawResponse,
    AsyncDeleteResourceWithRawResponse,
    DeleteResourceWithStreamingResponse,
    AsyncDeleteResourceWithStreamingResponse,
)
from .gemini import (
    GeminiResource,
    AsyncGeminiResource,
    GeminiResourceWithRawResponse,
    AsyncGeminiResourceWithRawResponse,
    GeminiResourceWithStreamingResponse,
    AsyncGeminiResourceWithStreamingResponse,
)
from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .openai import (
    OpenAIResource,
    AsyncOpenAIResource,
    OpenAIResourceWithRawResponse,
    AsyncOpenAIResourceWithRawResponse,
    OpenAIResourceWithStreamingResponse,
    AsyncOpenAIResourceWithStreamingResponse,
)
from .rerank import (
    RerankResource,
    AsyncRerankResource,
    RerankResourceWithRawResponse,
    AsyncRerankResourceWithRawResponse,
    RerankResourceWithStreamingResponse,
    AsyncRerankResourceWithStreamingResponse,
)
from .routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)
from .batches import (
    BatchesResource,
    AsyncBatchesResource,
    BatchesResourceWithRawResponse,
    AsyncBatchesResourceWithRawResponse,
    BatchesResourceWithStreamingResponse,
    AsyncBatchesResourceWithStreamingResponse,
)
from .bedrock import (
    BedrockResource,
    AsyncBedrockResource,
    BedrockResourceWithRawResponse,
    AsyncBedrockResourceWithRawResponse,
    BedrockResourceWithStreamingResponse,
    AsyncBedrockResourceWithStreamingResponse,
)
from .engines import (
    EnginesResource,
    AsyncEnginesResource,
    EnginesResourceWithRawResponse,
    AsyncEnginesResourceWithRawResponse,
    EnginesResourceWithStreamingResponse,
    AsyncEnginesResourceWithStreamingResponse,
)
from .global_ import (
    GlobalResource,
    AsyncGlobalResource,
    GlobalResourceWithRawResponse,
    AsyncGlobalResourceWithRawResponse,
    GlobalResourceWithStreamingResponse,
    AsyncGlobalResourceWithStreamingResponse,
)
from .threads import (
    ThreadsResource,
    AsyncThreadsResource,
    ThreadsResourceWithRawResponse,
    AsyncThreadsResourceWithRawResponse,
    ThreadsResourceWithStreamingResponse,
    AsyncThreadsResourceWithStreamingResponse,
)
from .customer import (
    CustomerResource,
    AsyncCustomerResource,
    CustomerResourceWithRawResponse,
    AsyncCustomerResourceWithRawResponse,
    CustomerResourceWithStreamingResponse,
    AsyncCustomerResourceWithStreamingResponse,
)
from .langfuse import (
    LangfuseResource,
    AsyncLangfuseResource,
    LangfuseResourceWithRawResponse,
    AsyncLangfuseResourceWithRawResponse,
    LangfuseResourceWithStreamingResponse,
    AsyncLangfuseResourceWithStreamingResponse,
)
from .provider import (
    ProviderResource,
    AsyncProviderResource,
    ProviderResourceWithRawResponse,
    AsyncProviderResourceWithRawResponse,
    ProviderResourceWithStreamingResponse,
    AsyncProviderResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .anthropic import (
    AnthropicResource,
    AsyncAnthropicResource,
    AnthropicResourceWithRawResponse,
    AsyncAnthropicResourceWithRawResponse,
    AnthropicResourceWithStreamingResponse,
    AsyncAnthropicResourceWithStreamingResponse,
)
from .responses import (
    ResponsesResource,
    AsyncResponsesResource,
    ResponsesResourceWithRawResponse,
    AsyncResponsesResourceWithRawResponse,
    ResponsesResourceWithStreamingResponse,
    AsyncResponsesResourceWithStreamingResponse,
)
from .vertex_ai import (
    VertexAIResource,
    AsyncVertexAIResource,
    VertexAIResourceWithRawResponse,
    AsyncVertexAIResourceWithRawResponse,
    VertexAIResourceWithStreamingResponse,
    AsyncVertexAIResourceWithStreamingResponse,
)
from .assemblyai import (
    AssemblyaiResource,
    AsyncAssemblyaiResource,
    AssemblyaiResourceWithRawResponse,
    AsyncAssemblyaiResourceWithRawResponse,
    AssemblyaiResourceWithStreamingResponse,
    AsyncAssemblyaiResourceWithStreamingResponse,
)
from .assistants import (
    AssistantsResource,
    AsyncAssistantsResource,
    AssistantsResourceWithRawResponse,
    AsyncAssistantsResourceWithRawResponse,
    AssistantsResourceWithStreamingResponse,
    AsyncAssistantsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .guardrails import (
    GuardrailsResource,
    AsyncGuardrailsResource,
    GuardrailsResourceWithRawResponse,
    AsyncGuardrailsResourceWithRawResponse,
    GuardrailsResourceWithStreamingResponse,
    AsyncGuardrailsResourceWithStreamingResponse,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)
from .fine_tuning import (
    FineTuningResource,
    AsyncFineTuningResource,
    FineTuningResourceWithRawResponse,
    AsyncFineTuningResourceWithRawResponse,
    FineTuningResourceWithStreamingResponse,
    AsyncFineTuningResourceWithStreamingResponse,
)
from .model_group import (
    ModelGroupResource,
    AsyncModelGroupResource,
    ModelGroupResourceWithRawResponse,
    AsyncModelGroupResourceWithRawResponse,
    ModelGroupResourceWithStreamingResponse,
    AsyncModelGroupResourceWithStreamingResponse,
)
from .moderations import (
    ModerationsResource,
    AsyncModerationsResource,
    ModerationsResourceWithRawResponse,
    AsyncModerationsResourceWithRawResponse,
    ModerationsResourceWithStreamingResponse,
    AsyncModerationsResourceWithStreamingResponse,
)
from .organization import (
    OrganizationResource,
    AsyncOrganizationResource,
    OrganizationResourceWithRawResponse,
    AsyncOrganizationResourceWithRawResponse,
    OrganizationResourceWithStreamingResponse,
    AsyncOrganizationResourceWithStreamingResponse,
)
from .eu_assemblyai import (
    EuAssemblyaiResource,
    AsyncEuAssemblyaiResource,
    EuAssemblyaiResourceWithRawResponse,
    AsyncEuAssemblyaiResourceWithRawResponse,
    EuAssemblyaiResourceWithStreamingResponse,
    AsyncEuAssemblyaiResourceWithStreamingResponse,
)

__all__ = [
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "OpenAIResource",
    "AsyncOpenAIResource",
    "OpenAIResourceWithRawResponse",
    "AsyncOpenAIResourceWithRawResponse",
    "OpenAIResourceWithStreamingResponse",
    "AsyncOpenAIResourceWithStreamingResponse",
    "EnginesResource",
    "AsyncEnginesResource",
    "EnginesResourceWithRawResponse",
    "AsyncEnginesResourceWithRawResponse",
    "EnginesResourceWithStreamingResponse",
    "AsyncEnginesResourceWithStreamingResponse",
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "CompletionsResource",
    "AsyncCompletionsResource",
    "CompletionsResourceWithRawResponse",
    "AsyncCompletionsResourceWithRawResponse",
    "CompletionsResourceWithStreamingResponse",
    "AsyncCompletionsResourceWithStreamingResponse",
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "ImagesResource",
    "AsyncImagesResource",
    "ImagesResourceWithRawResponse",
    "AsyncImagesResourceWithRawResponse",
    "ImagesResourceWithStreamingResponse",
    "AsyncImagesResourceWithStreamingResponse",
    "AudioResource",
    "AsyncAudioResource",
    "AudioResourceWithRawResponse",
    "AsyncAudioResourceWithRawResponse",
    "AudioResourceWithStreamingResponse",
    "AsyncAudioResourceWithStreamingResponse",
    "AssistantsResource",
    "AsyncAssistantsResource",
    "AssistantsResourceWithRawResponse",
    "AsyncAssistantsResourceWithRawResponse",
    "AssistantsResourceWithStreamingResponse",
    "AsyncAssistantsResourceWithStreamingResponse",
    "ThreadsResource",
    "AsyncThreadsResource",
    "ThreadsResourceWithRawResponse",
    "AsyncThreadsResourceWithRawResponse",
    "ThreadsResourceWithStreamingResponse",
    "AsyncThreadsResourceWithStreamingResponse",
    "ModerationsResource",
    "AsyncModerationsResource",
    "ModerationsResourceWithRawResponse",
    "AsyncModerationsResourceWithRawResponse",
    "ModerationsResourceWithStreamingResponse",
    "AsyncModerationsResourceWithStreamingResponse",
    "UtilsResource",
    "AsyncUtilsResource",
    "UtilsResourceWithRawResponse",
    "AsyncUtilsResourceWithRawResponse",
    "UtilsResourceWithStreamingResponse",
    "AsyncUtilsResourceWithStreamingResponse",
    "ModelResource",
    "AsyncModelResource",
    "ModelResourceWithRawResponse",
    "AsyncModelResourceWithRawResponse",
    "ModelResourceWithStreamingResponse",
    "AsyncModelResourceWithStreamingResponse",
    "ModelGroupResource",
    "AsyncModelGroupResource",
    "ModelGroupResourceWithRawResponse",
    "AsyncModelGroupResourceWithRawResponse",
    "ModelGroupResourceWithStreamingResponse",
    "AsyncModelGroupResourceWithStreamingResponse",
    "RoutesResource",
    "AsyncRoutesResource",
    "RoutesResourceWithRawResponse",
    "AsyncRoutesResourceWithRawResponse",
    "RoutesResourceWithStreamingResponse",
    "AsyncRoutesResourceWithStreamingResponse",
    "ResponsesResource",
    "AsyncResponsesResource",
    "ResponsesResourceWithRawResponse",
    "AsyncResponsesResourceWithRawResponse",
    "ResponsesResourceWithStreamingResponse",
    "AsyncResponsesResourceWithStreamingResponse",
    "BatchesResource",
    "AsyncBatchesResource",
    "BatchesResourceWithRawResponse",
    "AsyncBatchesResourceWithRawResponse",
    "BatchesResourceWithStreamingResponse",
    "AsyncBatchesResourceWithStreamingResponse",
    "RerankResource",
    "AsyncRerankResource",
    "RerankResourceWithRawResponse",
    "AsyncRerankResourceWithRawResponse",
    "RerankResourceWithStreamingResponse",
    "AsyncRerankResourceWithStreamingResponse",
    "FineTuningResource",
    "AsyncFineTuningResource",
    "FineTuningResourceWithRawResponse",
    "AsyncFineTuningResourceWithRawResponse",
    "FineTuningResourceWithStreamingResponse",
    "AsyncFineTuningResourceWithStreamingResponse",
    "CredentialsResource",
    "AsyncCredentialsResource",
    "CredentialsResourceWithRawResponse",
    "AsyncCredentialsResourceWithRawResponse",
    "CredentialsResourceWithStreamingResponse",
    "AsyncCredentialsResourceWithStreamingResponse",
    "VertexAIResource",
    "AsyncVertexAIResource",
    "VertexAIResourceWithRawResponse",
    "AsyncVertexAIResourceWithRawResponse",
    "VertexAIResourceWithStreamingResponse",
    "AsyncVertexAIResourceWithStreamingResponse",
    "GeminiResource",
    "AsyncGeminiResource",
    "GeminiResourceWithRawResponse",
    "AsyncGeminiResourceWithRawResponse",
    "GeminiResourceWithStreamingResponse",
    "AsyncGeminiResourceWithStreamingResponse",
    "CohereResource",
    "AsyncCohereResource",
    "CohereResourceWithRawResponse",
    "AsyncCohereResourceWithRawResponse",
    "CohereResourceWithStreamingResponse",
    "AsyncCohereResourceWithStreamingResponse",
    "AnthropicResource",
    "AsyncAnthropicResource",
    "AnthropicResourceWithRawResponse",
    "AsyncAnthropicResourceWithRawResponse",
    "AnthropicResourceWithStreamingResponse",
    "AsyncAnthropicResourceWithStreamingResponse",
    "BedrockResource",
    "AsyncBedrockResource",
    "BedrockResourceWithRawResponse",
    "AsyncBedrockResourceWithRawResponse",
    "BedrockResourceWithStreamingResponse",
    "AsyncBedrockResourceWithStreamingResponse",
    "EuAssemblyaiResource",
    "AsyncEuAssemblyaiResource",
    "EuAssemblyaiResourceWithRawResponse",
    "AsyncEuAssemblyaiResourceWithRawResponse",
    "EuAssemblyaiResourceWithStreamingResponse",
    "AsyncEuAssemblyaiResourceWithStreamingResponse",
    "AssemblyaiResource",
    "AsyncAssemblyaiResource",
    "AssemblyaiResourceWithRawResponse",
    "AsyncAssemblyaiResourceWithRawResponse",
    "AssemblyaiResourceWithStreamingResponse",
    "AsyncAssemblyaiResourceWithStreamingResponse",
    "AzureResource",
    "AsyncAzureResource",
    "AzureResourceWithRawResponse",
    "AsyncAzureResourceWithRawResponse",
    "AzureResourceWithStreamingResponse",
    "AsyncAzureResourceWithStreamingResponse",
    "LangfuseResource",
    "AsyncLangfuseResource",
    "LangfuseResourceWithRawResponse",
    "AsyncLangfuseResourceWithRawResponse",
    "LangfuseResourceWithStreamingResponse",
    "AsyncLangfuseResourceWithStreamingResponse",
    "ConfigResource",
    "AsyncConfigResource",
    "ConfigResourceWithRawResponse",
    "AsyncConfigResourceWithRawResponse",
    "ConfigResourceWithStreamingResponse",
    "AsyncConfigResourceWithStreamingResponse",
    "TestResource",
    "AsyncTestResource",
    "TestResourceWithRawResponse",
    "AsyncTestResourceWithRawResponse",
    "TestResourceWithStreamingResponse",
    "AsyncTestResourceWithStreamingResponse",
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "ActiveResource",
    "AsyncActiveResource",
    "ActiveResourceWithRawResponse",
    "AsyncActiveResourceWithRawResponse",
    "ActiveResourceWithStreamingResponse",
    "AsyncActiveResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "KeyResource",
    "AsyncKeyResource",
    "KeyResourceWithRawResponse",
    "AsyncKeyResourceWithRawResponse",
    "KeyResourceWithStreamingResponse",
    "AsyncKeyResourceWithStreamingResponse",
    "UserResource",
    "AsyncUserResource",
    "UserResourceWithRawResponse",
    "AsyncUserResourceWithRawResponse",
    "UserResourceWithStreamingResponse",
    "AsyncUserResourceWithStreamingResponse",
    "TeamResource",
    "AsyncTeamResource",
    "TeamResourceWithRawResponse",
    "AsyncTeamResourceWithRawResponse",
    "TeamResourceWithStreamingResponse",
    "AsyncTeamResourceWithStreamingResponse",
    "OrganizationResource",
    "AsyncOrganizationResource",
    "OrganizationResourceWithRawResponse",
    "AsyncOrganizationResourceWithRawResponse",
    "OrganizationResourceWithStreamingResponse",
    "AsyncOrganizationResourceWithStreamingResponse",
    "CustomerResource",
    "AsyncCustomerResource",
    "CustomerResourceWithRawResponse",
    "AsyncCustomerResourceWithRawResponse",
    "CustomerResourceWithStreamingResponse",
    "AsyncCustomerResourceWithStreamingResponse",
    "SpendResource",
    "AsyncSpendResource",
    "SpendResourceWithRawResponse",
    "AsyncSpendResourceWithRawResponse",
    "SpendResourceWithStreamingResponse",
    "AsyncSpendResourceWithStreamingResponse",
    "GlobalResource",
    "AsyncGlobalResource",
    "GlobalResourceWithRawResponse",
    "AsyncGlobalResourceWithRawResponse",
    "GlobalResourceWithStreamingResponse",
    "AsyncGlobalResourceWithStreamingResponse",
    "ProviderResource",
    "AsyncProviderResource",
    "ProviderResourceWithRawResponse",
    "AsyncProviderResourceWithRawResponse",
    "ProviderResourceWithStreamingResponse",
    "AsyncProviderResourceWithStreamingResponse",
    "CacheResource",
    "AsyncCacheResource",
    "CacheResourceWithRawResponse",
    "AsyncCacheResourceWithRawResponse",
    "CacheResourceWithStreamingResponse",
    "AsyncCacheResourceWithStreamingResponse",
    "GuardrailsResource",
    "AsyncGuardrailsResource",
    "GuardrailsResourceWithRawResponse",
    "AsyncGuardrailsResourceWithRawResponse",
    "GuardrailsResourceWithStreamingResponse",
    "AsyncGuardrailsResourceWithStreamingResponse",
    "AddResource",
    "AsyncAddResource",
    "AddResourceWithRawResponse",
    "AsyncAddResourceWithRawResponse",
    "AddResourceWithStreamingResponse",
    "AsyncAddResourceWithStreamingResponse",
    "DeleteResource",
    "AsyncDeleteResource",
    "DeleteResourceWithRawResponse",
    "AsyncDeleteResourceWithRawResponse",
    "DeleteResourceWithStreamingResponse",
    "AsyncDeleteResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "BudgetResource",
    "AsyncBudgetResource",
    "BudgetResourceWithRawResponse",
    "AsyncBudgetResourceWithRawResponse",
    "BudgetResourceWithStreamingResponse",
    "AsyncBudgetResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from .cancel import (
    CancelResource,
    AsyncCancelResource,
    CancelResourceWithRawResponse,
    AsyncCancelResourceWithRawResponse,
    CancelResourceWithStreamingResponse,
    AsyncCancelResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.fine_tuning import job_list_params, job_create_params, job_retrieve_params

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def cancel(self) -> CancelResource:
        return CancelResource(self._client)

    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        custom_llm_provider: Literal["openai", "azure", "vertex_ai"],
        model: str,
        training_file: str,
        hyperparameters: Optional[job_create_params.Hyperparameters] | NotGiven = NOT_GIVEN,
        integrations: Optional[List[str]] | NotGiven = NOT_GIVEN,
        seed: Optional[int] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        validation_file: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Creates a fine-tuning job which begins the process of creating a new model from
        a given dataset. This is the equivalent of POST
        https://api.openai.com/v1/fine_tuning/jobs

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/fine-tuning/create

        Example Curl:

        ```
        curl http://localhost:4000/v1/fine_tuning/jobs       -H "Content-Type: application/json"       -H "Authorization: Bearer sk-1234"       -d '{
            "model": "gpt-3.5-turbo",
            "training_file": "file-abc123",
            "hyperparameters": {
              "n_epochs": 4
            }
          }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/fine_tuning/jobs",
            body=maybe_transform(
                {
                    "custom_llm_provider": custom_llm_provider,
                    "model": model,
                    "training_file": training_file,
                    "hyperparameters": hyperparameters,
                    "integrations": integrations,
                    "seed": seed,
                    "suffix": suffix,
                    "validation_file": validation_file,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve(
        self,
        fine_tuning_job_id: str,
        *,
        custom_llm_provider: Literal["openai", "azure"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a fine-tuning job.

        This is the equivalent of GET
        https://api.openai.com/v1/fine_tuning/jobs/{fine_tuning_job_id}

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `fine_tuning_job_id`: The ID of the fine-tuning job to retrieve.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._get(
            f"/v1/fine_tuning/jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"custom_llm_provider": custom_llm_provider}, job_retrieve_params.JobRetrieveParams
                ),
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        custom_llm_provider: Literal["openai", "azure"],
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Lists fine-tuning jobs for the organization.

        This is the equivalent of GET
        https://api.openai.com/v1/fine_tuning/jobs

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `after`: Identifier for the last job from the previous pagination request.
        - `limit`: Number of fine-tuning jobs to retrieve (default is 20).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/fine_tuning/jobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "custom_llm_provider": custom_llm_provider,
                        "after": after,
                        "limit": limit,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            cast_to=object,
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def cancel(self) -> AsyncCancelResource:
        return AsyncCancelResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        custom_llm_provider: Literal["openai", "azure", "vertex_ai"],
        model: str,
        training_file: str,
        hyperparameters: Optional[job_create_params.Hyperparameters] | NotGiven = NOT_GIVEN,
        integrations: Optional[List[str]] | NotGiven = NOT_GIVEN,
        seed: Optional[int] | NotGiven = NOT_GIVEN,
        suffix: Optional[str] | NotGiven = NOT_GIVEN,
        validation_file: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Creates a fine-tuning job which begins the process of creating a new model from
        a given dataset. This is the equivalent of POST
        https://api.openai.com/v1/fine_tuning/jobs

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/fine-tuning/create

        Example Curl:

        ```
        curl http://localhost:4000/v1/fine_tuning/jobs       -H "Content-Type: application/json"       -H "Authorization: Bearer sk-1234"       -d '{
            "model": "gpt-3.5-turbo",
            "training_file": "file-abc123",
            "hyperparameters": {
              "n_epochs": 4
            }
          }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/fine_tuning/jobs",
            body=await async_maybe_transform(
                {
                    "custom_llm_provider": custom_llm_provider,
                    "model": model,
                    "training_file": training_file,
                    "hyperparameters": hyperparameters,
                    "integrations": integrations,
                    "seed": seed,
                    "suffix": suffix,
                    "validation_file": validation_file,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve(
        self,
        fine_tuning_job_id: str,
        *,
        custom_llm_provider: Literal["openai", "azure"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a fine-tuning job.

        This is the equivalent of GET
        https://api.openai.com/v1/fine_tuning/jobs/{fine_tuning_job_id}

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `fine_tuning_job_id`: The ID of the fine-tuning job to retrieve.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return await self._get(
            f"/v1/fine_tuning/jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"custom_llm_provider": custom_llm_provider}, job_retrieve_params.JobRetrieveParams
                ),
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        custom_llm_provider: Literal["openai", "azure"],
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Lists fine-tuning jobs for the organization.

        This is the equivalent of GET
        https://api.openai.com/v1/fine_tuning/jobs

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `after`: Identifier for the last job from the previous pagination request.
        - `limit`: Number of fine-tuning jobs to retrieve (default is 20).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/fine_tuning/jobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "custom_llm_provider": custom_llm_provider,
                        "after": after,
                        "limit": limit,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            cast_to=object,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_raw_response_wrapper(
            jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithRawResponse:
        return CancelResourceWithRawResponse(self._jobs.cancel)


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_raw_response_wrapper(
            jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithRawResponse:
        return AsyncCancelResourceWithRawResponse(self._jobs.cancel)


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_streamed_response_wrapper(
            jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithStreamingResponse:
        return CancelResourceWithStreamingResponse(self._jobs.cancel)


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_streamed_response_wrapper(
            jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithStreamingResponse:
        return AsyncCancelResourceWithStreamingResponse(self._jobs.cancel)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TeamUpdateMemberResponse"]


class TeamUpdateMemberResponse(BaseModel):
    team_id: str

    user_id: str

    max_budget_in_team: Optional[float] = None

    user_email: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    UtilTokenCounterResponse,
    UtilTransformRequestResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUtils:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_supported_openai_params(self, client: Hanzo) -> None:
        util = client.utils.get_supported_openai_params(
            model="model",
        )
        assert_matches_type(object, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_supported_openai_params(self, client: Hanzo) -> None:
        response = client.utils.with_raw_response.get_supported_openai_params(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = response.parse()
        assert_matches_type(object, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_supported_openai_params(self, client: Hanzo) -> None:
        with client.utils.with_streaming_response.get_supported_openai_params(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = response.parse()
            assert_matches_type(object, util, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_token_counter(self, client: Hanzo) -> None:
        util = client.utils.token_counter(
            model="model",
        )
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_token_counter_with_all_params(self, client: Hanzo) -> None:
        util = client.utils.token_counter(
            model="model",
            messages=[{}],
            prompt="prompt",
        )
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_token_counter(self, client: Hanzo) -> None:
        response = client.utils.with_raw_response.token_counter(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = response.parse()
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_token_counter(self, client: Hanzo) -> None:
        with client.utils.with_streaming_response.token_counter(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = response.parse()
            assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_transform_request(self, client: Hanzo) -> None:
        util = client.utils.transform_request(
            call_type="embedding",
            request_body={},
        )
        assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_transform_request(self, client: Hanzo) -> None:
        response = client.utils.with_raw_response.transform_request(
            call_type="embedding",
            request_body={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = response.parse()
        assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_transform_request(self, client: Hanzo) -> None:
        with client.utils.with_streaming_response.transform_request(
            call_type="embedding",
            request_body={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = response.parse()
            assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUtils:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_supported_openai_params(self, async_client: AsyncHanzo) -> None:
        util = await async_client.utils.get_supported_openai_params(
            model="model",
        )
        assert_matches_type(object, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_supported_openai_params(self, async_client: AsyncHanzo) -> None:
        response = await async_client.utils.with_raw_response.get_supported_openai_params(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = await response.parse()
        assert_matches_type(object, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_supported_openai_params(self, async_client: AsyncHanzo) -> None:
        async with async_client.utils.with_streaming_response.get_supported_openai_params(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = await response.parse()
            assert_matches_type(object, util, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_token_counter(self, async_client: AsyncHanzo) -> None:
        util = await async_client.utils.token_counter(
            model="model",
        )
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_token_counter_with_all_params(self, async_client: AsyncHanzo) -> None:
        util = await async_client.utils.token_counter(
            model="model",
            messages=[{}],
            prompt="prompt",
        )
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_token_counter(self, async_client: AsyncHanzo) -> None:
        response = await async_client.utils.with_raw_response.token_counter(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = await response.parse()
        assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_token_counter(self, async_client: AsyncHanzo) -> None:
        async with async_client.utils.with_streaming_response.token_counter(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = await response.parse()
            assert_matches_type(UtilTokenCounterResponse, util, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_transform_request(self, async_client: AsyncHanzo) -> None:
        util = await async_client.utils.transform_request(
            call_type="embedding",
            request_body={},
        )
        assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_transform_request(self, async_client: AsyncHanzo) -> None:
        response = await async_client.utils.with_raw_response.transform_request(
            call_type="embedding",
            request_body={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        util = await response.parse()
        assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_transform_request(self, async_client: AsyncHanzo) -> None:
        async with async_client.utils.with_streaming_response.transform_request(
            call_type="embedding",
            request_body={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            util = await response.parse()
            assert_matches_type(UtilTransformRequestResponse, util, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .budget_table import BudgetTable

__all__ = ["LiteLlmEndUserTable"]


class LiteLlmEndUserTable(BaseModel):
    blocked: bool

    user_id: str

    alias: Optional[str] = None

    allowed_model_region: Optional[Literal["eu", "us"]] = None

    default_model: Optional[str] = None

    litellm_budget_table: Optional[BudgetTable] = None
    """Represents user-controllable params for a LiteLLM_BudgetTable record"""

    spend: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["KeyBlockParams"]


class KeyBlockParams(TypedDict, total=False):
    key: Required[str]

    litellm_changed_by: Annotated[str, PropertyInfo(alias="litellm-changed-by")]
    """
    The litellm-changed-by header enables tracking of actions performed by
    authorized users on behalf of other users, providing an audit trail for
    accountability
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from .model import (
    ModelResource,
    AsyncModelResource,
    ModelResourceWithRawResponse,
    AsyncModelResourceWithRawResponse,
    ModelResourceWithStreamingResponse,
    AsyncModelResourceWithStreamingResponse,
)
from .update import (
    UpdateResource,
    AsyncUpdateResource,
    UpdateResourceWithRawResponse,
    AsyncUpdateResourceWithRawResponse,
    UpdateResourceWithStreamingResponse,
    AsyncUpdateResourceWithStreamingResponse,
)

__all__ = [
    "InfoResource",
    "AsyncInfoResource",
    "InfoResourceWithRawResponse",
    "AsyncInfoResourceWithRawResponse",
    "InfoResourceWithStreamingResponse",
    "AsyncInfoResourceWithStreamingResponse",
    "UpdateResource",
    "AsyncUpdateResource",
    "UpdateResourceWithRawResponse",
    "AsyncUpdateResourceWithRawResponse",
    "UpdateResourceWithStreamingResponse",
    "AsyncUpdateResourceWithStreamingResponse",
    "ModelResource",
    "AsyncModelResource",
    "ModelResourceWithRawResponse",
    "AsyncModelResourceWithRawResponse",
    "ModelResourceWithStreamingResponse",
    "AsyncModelResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import TypedDict

__all__ = ["SpendCalculateSpendParams"]


class SpendCalculateSpendParams(TypedDict, total=False):
    completion_response: Optional[object]

    messages: Optional[Iterable[object]]

    model: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .pass_through_generic_endpoint import PassThroughGenericEndpoint as PassThroughGenericEndpoint
from .pass_through_endpoint_response import PassThroughEndpointResponse as PassThroughEndpointResponse
from .pass_through_endpoint_list_params import PassThroughEndpointListParams as PassThroughEndpointListParams
from .pass_through_endpoint_create_params import PassThroughEndpointCreateParams as PassThroughEndpointCreateParams
from .pass_through_endpoint_delete_params import PassThroughEndpointDeleteParams as PassThroughEndpointDeleteParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .spend import (
    SpendResource,
    AsyncSpendResource,
    SpendResourceWithRawResponse,
    AsyncSpendResourceWithRawResponse,
    SpendResourceWithStreamingResponse,
    AsyncSpendResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["GlobalResource", "AsyncGlobalResource"]


class GlobalResource(SyncAPIResource):
    @cached_property
    def spend(self) -> SpendResource:
        return SpendResource(self._client)

    @cached_property
    def with_raw_response(self) -> GlobalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return GlobalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return GlobalResourceWithStreamingResponse(self)


class AsyncGlobalResource(AsyncAPIResource):
    @cached_property
    def spend(self) -> AsyncSpendResource:
        return AsyncSpendResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGlobalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncGlobalResourceWithStreamingResponse(self)


class GlobalResourceWithRawResponse:
    def __init__(self, global_: GlobalResource) -> None:
        self._global_ = global_

    @cached_property
    def spend(self) -> SpendResourceWithRawResponse:
        return SpendResourceWithRawResponse(self._global_.spend)


class AsyncGlobalResourceWithRawResponse:
    def __init__(self, global_: AsyncGlobalResource) -> None:
        self._global_ = global_

    @cached_property
    def spend(self) -> AsyncSpendResourceWithRawResponse:
        return AsyncSpendResourceWithRawResponse(self._global_.spend)


class GlobalResourceWithStreamingResponse:
    def __init__(self, global_: GlobalResource) -> None:
        self._global_ = global_

    @cached_property
    def spend(self) -> SpendResourceWithStreamingResponse:
        return SpendResourceWithStreamingResponse(self._global_.spend)


class AsyncGlobalResourceWithStreamingResponse:
    def __init__(self, global_: AsyncGlobalResource) -> None:
        self._global_ = global_

    @cached_property
    def spend(self) -> AsyncSpendResourceWithStreamingResponse:
        return AsyncSpendResourceWithStreamingResponse(self._global_.spend)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["BatchListParams"]


class BatchListParams(TypedDict, total=False):
    after: Optional[str]

    limit: Optional[int]

    provider: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .pass_through_generic_endpoint import PassThroughGenericEndpoint

__all__ = ["PassThroughEndpointResponse"]


class PassThroughEndpointResponse(BaseModel):
    endpoints: List[PassThroughGenericEndpoint]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomerRetrieveInfoParams"]


class CustomerRetrieveInfoParams(TypedDict, total=False):
    end_user_id: Required[str]
    """End User ID in the request parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["AnthropicResource", "AsyncAnthropicResource"]


class AnthropicResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AnthropicResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AnthropicResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AnthropicResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AnthropicResourceWithStreamingResponse(self)

    def create(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return self._post(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return self._get(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return self._put(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return self._delete(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def modify(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return self._patch(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAnthropicResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAnthropicResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAnthropicResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAnthropicResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncAnthropicResourceWithStreamingResponse(self)

    async def create(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return await self._post(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return await self._get(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return await self._put(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return await self._delete(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def modify(
        self,
        endpoint: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [Docs](https://docs.litellm.ai/docs/anthropic_completion)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not endpoint:
            raise ValueError(f"Expected a non-empty value for `endpoint` but received {endpoint!r}")
        return await self._patch(
            f"/anthropic/{endpoint}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AnthropicResourceWithRawResponse:
    def __init__(self, anthropic: AnthropicResource) -> None:
        self._anthropic = anthropic

        self.create = to_raw_response_wrapper(
            anthropic.create,
        )
        self.retrieve = to_raw_response_wrapper(
            anthropic.retrieve,
        )
        self.update = to_raw_response_wrapper(
            anthropic.update,
        )
        self.delete = to_raw_response_wrapper(
            anthropic.delete,
        )
        self.modify = to_raw_response_wrapper(
            anthropic.modify,
        )


class AsyncAnthropicResourceWithRawResponse:
    def __init__(self, anthropic: AsyncAnthropicResource) -> None:
        self._anthropic = anthropic

        self.create = async_to_raw_response_wrapper(
            anthropic.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            anthropic.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            anthropic.update,
        )
        self.delete = async_to_raw_response_wrapper(
            anthropic.delete,
        )
        self.modify = async_to_raw_response_wrapper(
            anthropic.modify,
        )


class AnthropicResourceWithStreamingResponse:
    def __init__(self, anthropic: AnthropicResource) -> None:
        self._anthropic = anthropic

        self.create = to_streamed_response_wrapper(
            anthropic.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            anthropic.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            anthropic.update,
        )
        self.delete = to_streamed_response_wrapper(
            anthropic.delete,
        )
        self.modify = to_streamed_response_wrapper(
            anthropic.modify,
        )


class AsyncAnthropicResourceWithStreamingResponse:
    def __init__(self, anthropic: AsyncAnthropicResource) -> None:
        self._anthropic = anthropic

        self.create = async_to_streamed_response_wrapper(
            anthropic.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            anthropic.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            anthropic.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            anthropic.delete,
        )
        self.modify = async_to_streamed_response_wrapper(
            anthropic.modify,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["KeyCheckHealthResponse", "LoggingCallbacks"]


class LoggingCallbacks(BaseModel):
    callbacks: Optional[List[str]] = None

    details: Optional[str] = None

    status: Optional[Literal["healthy", "unhealthy"]] = None


class KeyCheckHealthResponse(BaseModel):
    key: Optional[Literal["healthy", "unhealthy"]] = None

    logging_callbacks: Optional[LoggingCallbacks] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional

from .._models import BaseModel

__all__ = ["GuardrailListResponse", "Guardrail", "GuardrailLitellmParams"]


class GuardrailLitellmParams(BaseModel):
    guardrail: str

    mode: Union[str, List[str]]

    default_on: Optional[bool] = None


class Guardrail(BaseModel):
    guardrail_info: Optional[object] = None

    guardrail_name: str

    litellm_params: GuardrailLitellmParams
    """The returned LiteLLM Params object for /guardrails/list"""


class GuardrailListResponse(BaseModel):
    guardrails: List[Guardrail]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .cancel import (
    CancelResource,
    AsyncCancelResource,
    CancelResourceWithRawResponse,
    AsyncCancelResourceWithRawResponse,
    CancelResourceWithStreamingResponse,
    AsyncCancelResourceWithStreamingResponse,
)

__all__ = [
    "CancelResource",
    "AsyncCancelResource",
    "CancelResourceWithRawResponse",
    "AsyncCancelResourceWithRawResponse",
    "CancelResourceWithStreamingResponse",
    "AsyncCancelResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
]
